package production_system;

import javax.swing.*;
import java.awt.Dimension;
import operatorAssistant.agent.OperatorAssistant;
import java.awt.BorderLayout;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.RowSorter;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import javax.swing.table.TableColumn;
import java.util.ArrayList;
import java.util.LinkedList;
import scenarioStructures.*;

//creates a new window showing all generated scenarios
public class FieldScenariosFrame extends JFrame {

	private JPanel fieldPanel;
	
	public FieldScenariosFrame() {
		super("Field scenarios - Et scenario gyldig for hele tidsperioden");
		
		//creates the GUI
		initializeComponent();
	}//end constructor
	
	//Creates the GUI
	private void initializeComponent() {
	    setSize(720, 650);
	    setLocation(540, 100);
		fieldPanel = new JPanel();
	}//end initializeComponent()
	
	//getting hours-parameter from a ChooseProductionTargetFrame-object.
	//getting the operatorAssistant-agent to wait for a scenario to be selected.
	public void generateFieldScenarios(ChooseProductionTargetFrame chooseProductionTarget, OperatorAssistant operatorAssistant, int hours) {
		operatorAssistant.start(chooseProductionTarget, hours);
	}//end generateFieldScenarios()
	
	//Shows a window with all the generated field-scenarios.
	public void showGeneratedFieldScenarios(Scenarios fieldScenarios) {
	    Object[][] rows = new Object[fieldScenarios.getNumberOfScenarios()][8]; //8 columns to represent all needed parameters.
	    
		for(int i=0; i<fieldScenarios.getNumberOfScenarios(); i++) {
			OneCompositeScenario oneFieldScenario = (OneCompositeScenario)fieldScenarios.getScenario(i);
			
			int totalOil = oneFieldScenario.getTotalOilForWholeScenario();
			int totalGas = oneFieldScenario.getTotalGasForWholeScenario();
			int totalWater = oneFieldScenario.getTotalWaterForWholeScenario();
			int totalSand = oneFieldScenario.getTotalSandForWholeScenario();
			
			//ArrayList hourProductions = oneFieldScenario.getProductionHourFromHour();
			
			double minOilWasteRatio = -1;
			double maxOilWasteRatio = -1;
			double averageOilWasteRatio = -1;
			
			boolean firstRound = true;
			
			averageOilWasteRatio = (double)totalOil/((double)totalGas+(double)totalWater+(double)totalSand);
			
			for(int x=0;x<oneFieldScenario.getNumberOfHours(); x++) {
				double int_hourOilProd = (double)oneFieldScenario.getHourProduction( (x+1), "oil");
				double int_hourGasProd = (double)oneFieldScenario.getHourProduction( (x+1), "gas");
				double int_hourWaterProd = (double)oneFieldScenario.getHourProduction( (x+1), "water");
				double int_hourSandProd = (double)oneFieldScenario.getHourProduction( (x+1), "sand");
				
				if(firstRound){
					
					minOilWasteRatio = int_hourOilProd/(int_hourGasProd+int_hourWaterProd+int_hourSandProd);
					maxOilWasteRatio = int_hourOilProd/(int_hourGasProd+int_hourWaterProd+int_hourSandProd);
					
					firstRound = false;
				}
				else{
					
					if( minOilWasteRatio>(int_hourOilProd/(int_hourGasProd+int_hourWaterProd+int_hourSandProd) ) ){
						minOilWasteRatio = int_hourOilProd/(int_hourGasProd+int_hourWaterProd+int_hourSandProd);
					}//end if
					
					if( maxOilWasteRatio<(int_hourOilProd/(int_hourGasProd+int_hourWaterProd+int_hourSandProd) ) ){
						maxOilWasteRatio = int_hourOilProd/(int_hourGasProd+int_hourWaterProd+int_hourSandProd);
					}//end if
				}//end if-else
			}//end for-loop
			
			rows[i][0] = new Integer(i+1); //scenario id er plassen den har i ArrayListen, slik at den lett kan hentes igjen seinere.
			rows[i][1] = new Integer(totalOil);
			rows[i][2] = new Integer(totalGas);
			rows[i][3] = new Integer(totalWater);
			rows[i][4] = new Integer(totalSand);
			rows[i][5] = new Double(maxOilWasteRatio);
			rows[i][6] = new Double(minOilWasteRatio);
			rows[i][7] = new Double(averageOilWasteRatio);	
			
		}//end for-loop

		
	    /* Specify column names */
	    String columns[] = { "Scenario Id", "Oil", "Gas", "Water", "Sand", "Maximum Oil/Waste ratio", "Minimum Oil/Waste ratio", "Average Oil/Waste ratio" };
	    
	    /* Create a TableModel */
	    TableModel model = new DefaultTableModel(rows, columns) {
	    	public Class getColumnClass(int column) {
	    		Class returnValue;
	    		if ((column >= 0) && (column < getColumnCount())) {
	    			returnValue = getValueAt(0, column).getClass();
	    		} 
	    		else {
	    			returnValue = Object.class;
	    		}//end if-else
	    		return returnValue;
	    	}
	    };

	    JTable table = new JTable(model);

	    // Disable auto resizing
	    table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	    
	    TableColumn col1 = table.getColumnModel().getColumn(0);
	    TableColumn col2 = table.getColumnModel().getColumn(1);
	    TableColumn col3 = table.getColumnModel().getColumn(2);
	    TableColumn col4 = table.getColumnModel().getColumn(3);
	    TableColumn col5 = table.getColumnModel().getColumn(4);
	    TableColumn col6 = table.getColumnModel().getColumn(5);
	    TableColumn col7 = table.getColumnModel().getColumn(6);
	    TableColumn col8 = table.getColumnModel().getColumn(7);
	    
	    col1.setPreferredWidth(70);
	    col2.setPreferredWidth(50);
	    col3.setPreferredWidth(50);
	    col4.setPreferredWidth(50);
	    col5.setPreferredWidth(50);
	    col6.setPreferredWidth(140);
	    col7.setPreferredWidth(140);
	    col8.setPreferredWidth(140);
	    
	    //RowSorter<TableModel> sorter = new TableRowSorter<TableModel>(model);
	    RowSorter sorter = new TableRowSorter(model);
	    table.setRowSorter(sorter);

	    JScrollPane pane = new JScrollPane(table);
	    Dimension dim = new Dimension(720, 650);
	    pane.setPreferredSize(dim);
	    fieldPanel.add(pane);
	    this.add(fieldPanel, BorderLayout.CENTER);
	    this.setVisible(true);

	}//end showGeneratedFieldScenarios()


}//end class
