package scenarioStructures;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.LinkedList;

public class OneCompositeScenario extends OneScenario implements Serializable {

	private ArrayList scenariosInvolvedHourByHour;//contains an hour by hour list, with each hour containing a Scenarios-object containing all scenarios involved that hour.
	private int hoursValid;
	
	//constructor
	public OneCompositeScenario() {
		scenariosInvolvedHourByHour = new ArrayList();
	}//end constructor
	
	//constructor
	public OneCompositeScenario(int hoursValid) {
		scenariosInvolvedHourByHour = new ArrayList();
		this.hoursValid = hoursValid;
		
		for(int i=0; i<hoursValid; i++){
			Scenarios scenariosInvolvedThisHour = new Scenarios();
			scenariosInvolvedHourByHour.add(scenariosInvolvedThisHour);
		}//end for-loop
	}//end constructor
	
	//Get all scenarios inluded in a composite scenario, in a specified hour.
	public Scenarios getInvolvedScenarios(int hour) {
		return (Scenarios)scenariosInvolvedHourByHour.get(hour-1);
	}//end getInvolvedScenarios()
	
	//Get the number of scenarios inluded in a composite scenario, in a specified hour.
	public int getNumberOfInvolvedScenarios(int hour) {
		if(scenariosInvolvedHourByHour.size()==0) {
			return 0;
		}
		else {
			Scenarios scenariosThisHour = (Scenarios)scenariosInvolvedHourByHour.get(hour-1);
			return scenariosThisHour.getNumberOfScenarios();			
		}
	}//end getNumberOfInvolvedScenarios()
	
	//This method clones scenariosInvolvedHourByHour into the OneCompositeScenario-object given as parameter.
	public OneCompositeScenario cloneScenariosInvolvedHourByHour(OneCompositeScenario cloneOneCompositeScenario) {
		//Run through every hour
		for(int i=0; i<scenariosInvolvedHourByHour.size(); i++) {
			Scenarios temp = (Scenarios)scenariosInvolvedHourByHour.get(i);
			
			//Run through all involved scenarios in this composite scenario
			for(int x=0; x<temp.getNumberOfScenarios(); x++) {
				OneScenario oneScenario = (OneScenario)temp.getScenario(x);
				cloneOneCompositeScenario.buildCompositeScenario(oneScenario, i+1);
			}//end for-loop
			
		}//end for-loop

		return cloneOneCompositeScenario;
	}//end cloneScenariosInvolvedHourByHour()
	
	//This method adds a scenario to be involved in a composite scenario, 
	//in a specified hour in that composite scenario.
	public void buildCompositeScenario(OneScenario oneScenario, int hour){
		
		if(hour>scenariosInvolvedHourByHour.size()){//generate new hour
			Scenarios scenariosInvolvedThisHour = new Scenarios();
			scenariosInvolvedThisHour.addScenario(oneScenario);			
			scenariosInvolvedHourByHour.add(scenariosInvolvedThisHour);
		}
		else {//hour already exists
			Scenarios scenariosInvolvedThisHour = (Scenarios)scenariosInvolvedHourByHour.get(hour-1);
			scenariosInvolvedThisHour.addScenario(oneScenario);
		}//end if-else
		
		int oil = oneScenario.getHourProduction(hour, "oil");
		int gas = oneScenario.getHourProduction(hour, "gas");
		int water = oneScenario.getHourProduction(hour, "water");
		int sand = oneScenario.getHourProduction(hour, "sand");
		this.updateHourFromHourProduction(hour, new Integer(oil),  new Integer(gas),  new Integer(water),  new Integer(sand) );//Add production to the specified hour. This method will also add the production to the total production of the scenario.
	}//end buildOneSubseaTemplateScenario()
	
	//This method adds a scenario to be involved in a composite scenario, 
	//for every hour in that composite scenario.
	public void buildCompositeScenario(OneCompositeScenario oneScenario){
		//g igjennom hver time. Legge samme oneScenario for hver time
		for(int i=1; i<=hoursValid; i++) {
			int oil = oneScenario.getHourProduction(i, "oil");
			int gas = oneScenario.getHourProduction(i, "gas");
			int water = oneScenario.getHourProduction(i, "water");
			int sand = oneScenario.getHourProduction(i, "sand");
			this.updateHourFromHourProduction(i, new Integer(oil), new Integer(gas), new Integer(water), new Integer(sand));//Add production to the specified hour. This method will also add the production to the total production of the scenario.
			Scenarios scenariosInvolvedThisHour = (Scenarios)scenariosInvolvedHourByHour.get(i-1);
			scenariosInvolvedThisHour.addScenario(oneScenario);	
		}//end for-loop
	}//end buildOneSubseaTemplateScenario()
	
	private void clearScenariosInvolved(int hour){
		Scenarios emptyScenarios = new Scenarios();
		scenariosInvolvedHourByHour.set(hour-1, emptyScenarios);
	}//clearScenariosInvolved()
	
	//Clear hour-production, subtract the total composite scenario production, and clear the scenarios involved in the composite scenario.
	public void clearProduction(int hour) {
		emptyHourProduction(hour);//empty production in a specified hour, and subtract the production on the total production as well.
		clearScenariosInvolved(hour);//delete the scenarios involved this specified hour.
	}//end clearProduction()
	
	
}//end class
