package scenarioStructures;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.LinkedList;
import scenarioStructures.*;

public class OneScenario implements Serializable {

	private String unitName; //wellName or subseaTemplateName or geoAreaName or fieldName that the scenario belongs to.
	private int scenarioId; //the id on this scenario
	private ArrayList listForWholeScenario; //contains amount of production hour by hour. Each element contains a LinkedList containing the hour-production.
	private int totalOilForWholeScenario = 0; //total amount of oil for this scenario, for the whole time period.
	private int totalGasForWholeScenario = 0; //total amount of gas for this scenario, for the whole time period.
	private int totalWaterForWholeScenario = 0; //total amount of water for this scenario, for the whole time period.
	private int totalSandForWholeScenario = 0; //total amount of sand for this scenario, for the whole time period.
	
	//constructor
	public OneScenario()  {
		listForWholeScenario = new ArrayList();
	}//end constructor

	/*	
	//This method clones listForWholeScenario into the OneCompositeScenario-object given as parameter.
	public OneCompositeScenario cloneProductionInvolvedHourByHour(OneCompositeScenario cloneOneCompositeScenario) {
		//ArrayList tempList = cloneOneCompositeScenario.getHourByHourProductionList();
		
		//Run through every hour
		for(int i=0; i<listForWholeScenario.size(); i++) {
			LinkedList temp = (LinkedList)listForWholeScenario.get(i);

			Integer oil = (Integer)temp.get(0);
			Integer gas = (Integer)temp.get(1);
			Integer water = (Integer)temp.get(2);
			Integer sand = (Integer)temp.get(3);
			
			cloneOneCompositeScenario.updateHourFromHourProduction(i, new Integer( oil.intValue() ), new Integer( gas.intValue() ), new Integer( water.intValue() ), new Integer( sand.intValue() ));	
		}//end for-loop

		return cloneOneCompositeScenario;
	}//end cloneScenariosInvolvedHourByHour()
	*/
	
	public ArrayList getHourByHourProductionList() {
		return listForWholeScenario;
	}//end getHourByHourProductionList()
	
	public void setHourByHourProductionList(ArrayList hourByHourProductionList) {
		listForWholeScenario = hourByHourProductionList;
	}//end setHourByHourProductionList()
	
	//set the name on the unit
	public void setUnitName(String unitName){
		this.unitName = unitName;
	}//end setUnitName()
	
	//get the name on the unit
	public String getUnitName(){
		return unitName;
	}//end getUnitName()
	
	//set the scenario id
	public void setScenarioId(int scenarioId){
		this.scenarioId = scenarioId;
	}//end setScenarioId()
	
	
	//get the scenario id
	public int getScenarioId(){
		return scenarioId;
	}//end getScenarioId()
	

	//get the number of hours this scenario is valid for.
	public int getNumberOfHours() {
		return listForWholeScenario.size();
	}//end getNumberOfHours()
	
	public int getTotalOilForWholeScenario(){
		return totalOilForWholeScenario;
	}//end getTotalOilForWholeScenario()
	
	public int getTotalGasForWholeScenario(){
		return totalGasForWholeScenario;
	}//end getTotalGasForWholeScenario()
	
	public int getTotalWaterForWholeScenario(){
		return totalWaterForWholeScenario;
	}//end getTotalWaterForWholeScenario()
	
	public int getTotalSandForWholeScenario(){
		return totalSandForWholeScenario;
	}//end getTotalSandForWholeScenario()
	
	public void addNextHourProduction(String oil, String gas, String water, String sand) {
		LinkedList hourProduction = new LinkedList();
		Integer integer_oil = new Integer(oil);
		Integer integer_gas = new Integer(gas);
		Integer integer_water = new Integer(water);
		Integer integer_sand = new Integer(sand);
		
		hourProduction.add(integer_oil);
		hourProduction.add(integer_gas);
		hourProduction.add(integer_water);
		hourProduction.add(integer_sand);
		
		updateTotalProduction( integer_oil.intValue(), integer_gas.intValue(), integer_water.intValue(), integer_sand.intValue() );
		
		listForWholeScenario.add(hourProduction);
	}//end addNextHourProduction()	
	
	private void updateTotalProduction(int oil, int gas, int water, int sand){
		totalOilForWholeScenario = totalOilForWholeScenario + oil;
		totalGasForWholeScenario = totalGasForWholeScenario + gas;
		totalWaterForWholeScenario = totalWaterForWholeScenario + water;
		totalSandForWholeScenario = totalSandForWholeScenario + sand;
	}//end updateTotalProduction()
	
	//Get the amount of production of a specified substance from a specified hour.
	//Substance parameters are: "oil", "gas", "water" or "sand".
	public int getHourProduction(int hour, String substance) {
		LinkedList hourProd = (LinkedList)listForWholeScenario.get(hour-1);
		
		if(substance.equals("oil")) {
			Integer oil = (Integer)hourProd.get(0);
			return oil.intValue();
		}
		else if(substance.equals("gas")) {
			Integer gas = (Integer)hourProd.get(1);
			return gas.intValue();
		}
		else if(substance.equals("water")) {
			Integer water = (Integer)hourProd.get(2);
			return water.intValue();
		}
		else if(substance.equals("sand")) {
			Integer sand = (Integer)hourProd.get(3);
			return sand.intValue();
		}
		else {
			return -1;
		}//end if-else
	}//end getHourProduction()	
	
	//This method updates the specified hour with specified production.
	//The production is also added to the total production of this scenario.
	public void updateHourFromHourProduction(int hour, Integer oil, Integer gas, Integer water, Integer sand){	
		if(listForWholeScenario.size()<hour) {//if there is NO LinkedList for the specified hour.			
			LinkedList newHour = new LinkedList();
			newHour.add(oil);
			newHour.add(gas);
			newHour.add(water);
			newHour.add(sand);
			listForWholeScenario.add(newHour);
		}
		else {//if there IS a LinkedList for the specified hour.
			LinkedList hourProduction = (LinkedList)listForWholeScenario.get(hour-1);
			Integer oldOilValue = (Integer)hourProduction.get(0);
			Integer oldGasValue = (Integer)hourProduction.get(1);
			Integer oldWaterValue = (Integer)hourProduction.get(2);
			Integer oldSandValue = (Integer)hourProduction.get(3);
			int int_oldOilValue = oldOilValue.intValue();
			int int_oldGasValue = oldGasValue.intValue();
			int int_oldWaterValue = oldWaterValue.intValue();
			int int_oldSandValue = oldSandValue.intValue();
			
			hourProduction.clear();
			hourProduction.add(0, new Integer(int_oldOilValue + oil.intValue() ) );
			hourProduction.add(1, new Integer(int_oldGasValue + gas.intValue() ) );
			hourProduction.add(2, new Integer(int_oldWaterValue + water.intValue() ) );
			hourProduction.add(3, new Integer(int_oldSandValue + sand.intValue() ) );
			listForWholeScenario.set((hour-1), hourProduction);			
		}//end if
		
		updateTotalProduction(oil.intValue(), gas.intValue(), water.intValue(), sand.intValue());
		
	}//end updateHourFromHourProduction()
	
	//This method empty production in a specified hour, and subtract the production on the total production as well.
	public void emptyHourProduction(int hour){
		LinkedList hourProduction = (LinkedList)listForWholeScenario.get(hour-1);
		Integer oil = (Integer)hourProduction.get(0);
		Integer gas = (Integer)hourProduction.get(1);
		Integer water = (Integer)hourProduction.get(2);
		Integer sand = (Integer)hourProduction.get(3);
		updateTotalProduction( -oil.intValue(), -gas.intValue(), -water.intValue(), -sand.intValue() );
		hourProduction.clear();
		hourProduction.add(0, new Integer(0) );
		hourProduction.add(1, new Integer(0) );
		hourProduction.add(2, new Integer(0) );
		hourProduction.add(3, new Integer(0) );		
		listForWholeScenario.set((hour-1), hourProduction);
	}//end emptyHourProduction()
	
}//end class
