package production_system;

import subseaTemplate.java.*;
import well.java.*;
import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import java.util.ArrayList;
import java.util.LinkedList;
import scenarioStructures.*;

//Creates the window showing the well-settings for specified scenario, hour by hour.
public class ChokeSettingsFrame extends JFrame {

	private int numberOfGeoAreas = 2;
	private int numberOfSubseaTemplates = 4;
	private int totalNumberOfWells = 12;
	private JPanel panel_chokeSettings;

	//constructor
	public ChokeSettingsFrame(OneCompositeScenario oneFieldScenario, int hoursValidity) {
		super("Field scenario - Well settings");
		initializeComponent();
		showGeneratedFieldScenarios(oneFieldScenario, hoursValidity);
	}//end constructor
	
	private void initializeComponent() {
	    setSize(500, 300);
	    setLocation(540, 100);
	    panel_chokeSettings = new JPanel();
	}//end initializeComponent()
	
	//Create the table with well-settings hour by hour for all wells, and show in the GUI.
	public void showGeneratedFieldScenarios(OneCompositeScenario oneFieldScenario, int hoursValidity) {
		//System.out.println("Antall geoArea-scenarioer involvert: "+oneFieldScenario.getNumberOfInvolvedScenarios(1));
	    Object[][] rows = new Object[totalNumberOfWells][hoursValidity+1];
	    int nextIndex = 0;
	    
	    //System.out.println("TEST 1: "+totalNumberOfWells+", hoursValidity+1: "+(hoursValidity+1));
		//run through all GeoArea-scenarios that are included in the Field-scenario.
	    for(int hour=1; hour<=hoursValidity; hour++) {
	    	Scenarios involvedGeoAreaScenarios = oneFieldScenario.getInvolvedScenarios(hour);
	    	//rows[0][hour] = 
	    	nextIndex = 0;//a new hours starts, therefore restart the index, so the next well productions will belong to the same wells in the table.

	    	for(int i=0; i<involvedGeoAreaScenarios.getNumberOfScenarios(); i++) {
	    		OneCompositeScenario oneGeoAreaScenario = (OneCompositeScenario)involvedGeoAreaScenarios.getScenario(i);
	    		Scenarios involvedSubseaTemplateScenarios = oneGeoAreaScenario.getInvolvedScenarios(hour);

	    		for(int x=0; x<involvedSubseaTemplateScenarios.getNumberOfScenarios(); x++) {
		    		OneCompositeScenario oneSubseaTemplateScenario = (OneCompositeScenario)involvedSubseaTemplateScenarios.getScenario(x);
		    		Scenarios involvedWellScenarios = oneSubseaTemplateScenario.getInvolvedScenarios(hour);

		    		//run through all well-scenarios for specified hour
		    		for(int y=0; y<involvedWellScenarios.getNumberOfScenarios(); y++) {
		    			OneWellScenario oneWellScenario = (OneWellScenario)involvedWellScenarios.getScenario(y);
			    		String wellName = oneWellScenario.getUnitName();
			    		Integer chokeLevel = new Integer( oneWellScenario.getChokeLevel() );
			    		/*
			    		System.out.println("wellName: "+wellName);
			    		System.out.println("chokeLevel: "+chokeLevel);
			    		System.out.println("y: "+y);
			    		System.out.println("hour: "+hour);
			    		*/
			    		rows[y+nextIndex][0] = wellName;
			    		rows[y+nextIndex][hour] = chokeLevel;
		    		}//end for-loop
		    		nextIndex = nextIndex + 3;//add three so the index will be correct, since each subseaTemplates contains three wells each.
	    		}//end for-loop
	    	}//end for-loop
	    }//end for-loop
	    
	    /* Specify column names */
	    String columns[] = new String[hoursValidity+1];  
	    columns[0]="Well id";

	    for(int i=1;i<=hoursValidity;i++){
	    	columns[i] = "Hour "+String.valueOf(i);
	    }//end for-loop

	    JTable table = new JTable(rows, columns );
	    JScrollPane pane = new JScrollPane(table);
	    Dimension dim = new Dimension(500, 300);
	    pane.setPreferredSize(dim);
	    panel_chokeSettings.add(pane);
	    this.add(panel_chokeSettings, BorderLayout.CENTER);
	    this.setVisible(true);    
	}//end showGeneratedFieldScenarios()
	
}//end class
